#include "ImporterDicom.h"
#include "LoaderDicom.h"
#include "vrc_ImporterDicom.cpp"

ImporterDicom::ImporterDicom(Plugin & pluPlugin) : PluginInstance(pluPlugin)
{
	_observerFile.connect(this, &ImporterDicom::filenameChanged);
	_observerSerie.connect(this, &ImporterDicom::serieChanged);

	GetPlugin().GetProperty("Filename").require(Variant::TypeFilename("","*.*"));
	GetPlugin().GetProperty("Filename").addObserver(&_observerFile);

	if (!GetPlugin().HasProperty("Series")) 
		GetPlugin().GetProperty("Series") = Variant::TypeOption();
	GetPlugin().GetProperty("Series").addObserver(&_observerSerie);

	if (!GetPlugin().HasProperty("Slice")) // Volume or Time step
		GetPlugin().GetProperty("Slice") = Variant::TypeOption();

	// elastography property
	if (!GetPlugin().HasProperty("Elastography"))
		GetPlugin().GetProperty("Elastography") = Variant::TypeOption();
	GetPlugin().GetProperty("Elastography")[0] = std::string("Disabled");
	GetPlugin().GetProperty("Elastography")[1] = std::string("Elastography");
	GetPlugin().GetProperty("Elastography")[2] = std::string("B-Mode");
}

ImporterDicom::~ImporterDicom()
{
}

Loader* ImporterDicom::load(Resource & resResource)
{
	Loader *pLoader = NULL;
	try {
		pLoader = Loader::create<LoaderDicom, ImporterDicom>(resResource, *this);
		return pLoader;
	} catch (...) {
		if (pLoader) {
			delete pLoader;
			pLoader = NULL;
		}
	}
	return new Loader();
}

void ImporterDicom::filenameChanged(const Variant & variant, const Observable::Event & event)
{
	showSeries(true);
}

void ImporterDicom::serieChanged(const Variant & variant, const Observable::Event & event)
{
	//	prevent listing of slices while adding serie values
	if (!_serieLocked)
		showSlices();
}

void ImporterDicom::showSlices() {
	//	get selected serie
	gdcm::FileList *fileList = _serieHelper->GetFirstSingleSerieUIDFileSet();
	for (int i=0; fileList; i++) {
		std::stringstream ss;
		ss << (*fileList)[0]->GetEntryString(0x0008, 0x103E);
		ss << " (" << fileList->size() << " slides)";
		if (GetPlugin().GetProperty("Series") == ss.str())
			break;
		fileList = _serieHelper->GetNextSingleSerieUIDFileSet();
	}

	/*
		Make a list of available slices in a serie.
		Checks repeating "Slice Location".
	*/
	GetPlugin().GetProperty("Slice") = Variant::TypeOption();
	std::string firstSliceLocation = (*fileList)[0]->GetEntryString(0x0020, 0x1041);
	bool markSlice = true;
	for (int i=0,j=0; i<fileList->size(); i++) {
		std::string sliceLocation = (*fileList)[i]->GetEntryString(0x0020, 0x1041);
		if (firstSliceLocation != sliceLocation)
			markSlice = true;
		if (firstSliceLocation == sliceLocation && markSlice) {
			std::stringstream ss;
			ss << j + 1;
			GetPlugin().GetProperty("Slice")[j++] = ss.str();
			markSlice = false;
		}
	}
}

void ImporterDicom::showSeries(const bool clearList)
{
	_serieLocked = true;
	// clear list
	if (clearList)
		GetPlugin().GetProperty("Series") = Variant::TypeOption();

	// open files (Series values)
	_serieHelper = gdcm::SerieHelper::New();
	const Variant & files = GetPlugin().GetProperty("Filename");
	for (int i=0; i<files.GetSize(); i++)
		_serieHelper->AddFileName(std::string(files[i]));

	// set Serie description from 0x0008, 0x103E
	gdcm::FileList *fileList = _serieHelper->GetFirstSingleSerieUIDFileSet();
	for (int i=0; fileList; i++) {
		std::stringstream ss;
		ss << (*fileList)[0]->GetEntryString(0x0008, 0x103E);
		ss << " (" << fileList->size() << " slides)";
		GetPlugin().GetProperty("Series")[i] = ss.str();
		fileList = _serieHelper->GetNextSingleSerieUIDFileSet();
	}
	_serieLocked = false;
	showSlices();
}

DEFINE_IMPORTERPLUGIN(ImporterDicom,"DICOM Importer (*.*)",ImporterDicom_png)