#pragma once

#include "volumeshop.h"

#include "Renderer.h"
#include "RendererPlugin.h"
#include "PluginInstance.h"
#include "Font.h"
#include <gdcm.h>
#include <string>

/*
	Class which holds a DICOM element
*/
class DICOMItem
{
public:
	DICOMItem(int group, int element) : group(group), element(element) {}
	int group;
	int element;
	std::string getValue();
};

/*
	A text box which holds DICOM element
*/
struct box {
	float x, y, width, height;
	unsigned int color;
	int depth;
	std::vector<DICOMItem> item;
};


class RendererLabel : public Renderer, public PluginInstance
{
public:
	RendererLabel(Plugin & pluPlugin);
	virtual void Volumechanged(const Variant & variant, const Observable::Event & event);
	virtual void DICOMchanged(const Variant & variant, const Observable::Event & event);
	virtual void snapToVolumeChanged(const Variant & variant, const Observable::Event & event);
	virtual ~RendererLabel();
	virtual void display(Canvas & canCanvas);
	virtual void button(const Controller & conController);
	virtual void cursor(const Controller & conController);
	virtual void reshape(const unsigned int width, const unsigned int height);
protected:
	void updated(const Variant & varVariant, const Observable::Event & eveEvent);
private:
	void setDICOMList();
	bool hitTest(const Vector position, struct box *rect);
	void displayTextBoxes();
	void snapTextBoxes();
	void alignTextBoxes();

	ModifiedObserver _observerVolume;
	ModifiedObserver _observerDICOM;
	ModifiedObserver _observerSnap;
	gdcm::File *_file;
	unsigned int _width;
	unsigned int _height;
	bool _updateDICOMProperty;
	std::vector<gdcm::DictEntry*> _elementList;

	float fontHeight;

	std::vector<struct box> boxes;
	struct box *boxSelected;
	float _xOffset, _yOffset;
	float boxMargin;
};

DECLARE_RENDERERPLUGIN(RendererLabel)